<?php

require_once "ArrayPieceQuantik.php";
class PlateauQuantik
{
    
    const NBROWS = 4; //le plateau contient 4 lignes
    const NBCOLS = 4; //et 4 colones
    const NW = 0; //le numéro de la zone qui se situe au nord-ouest du plateau
    const NE = 1; //le numéro de la zone qui se situe au nord-est du plateau
    const SW = 2; //le numéro de la zone qui se situe au sud-ouest du plateau
    const SE = 3;  //le numéro de la zone qui se situe au sud-est du plateau
    public $cases ; //les cases du tableau

    //le constructeur
    public function __construct() {
            $this->cases = array();
            for ($i=0; $i <self::NBROWS ; $i++) {
                $this->cases[$i] = array();
                for ($j = 0; $j < self::NBCOLS; $j++) {
                    $this->setPiece($i, $j, PieceQuantik::initVoid());
                }
            }
    }

    //cette methode permet de recuperer la piece qui se trouve à la ligne rowNum et à la colonne colNum
    public function getPiece(int $rowNum, int $colNum) : PieceQuantik{
        return $this->cases[$rowNum][$colNum];
    }

    //cette methode permet de remplacer la piece qui se trouve au coordonnées rowNum et colNum avec la piece p
    public function setPiece(int $rowNum, int $colNum,PieceQuantik $p ) {
        $this->cases[$rowNum][$colNum]= $p;
    }

    //cette methode permet de récuperer les pieces qui sont dans la ligne numRow du plateau
    //elle les met dans un ArrayPieceQuantik puis retourne ce tableau
    public function getRow(int $numRow) : ArrayPieceQuantik{
        $t = new ArrayPieceQuantik();
        for($i = 0; $i < self::NBROWS; $i++){
            $t->addPieceQuantik($this->cases[$numRow][$i]);
        }
        return $t;
    }

    //cette methode permet de récuperer les pieces qui sont dans la colonne numRow du plateau
    //elle les met dans un ArrayPieceQuantik puis retourne ce tableau
    public function getCol(int $numCol) : ArrayPieceQuantik{
        $t = new ArrayPieceQuantik();
        for($i = 0; $i <self::NBCOLS; $i++){
            $t->addPieceQuantik($this->cases[$i][$numCol]);
        }
        return $t;
    }

    //cette methode permet de récuperer les pieces de la zone dir du plateau
    //elle recupere les 4 pieces d'une zone et il les met dans un ArrayPieceQuantik puis retourne ce tableau
    
    public function getCorner(int $dir) : ArrayPieceQuantik{
        $t = new ArrayPieceQuantik();
        if($dir == self::NW){//les pieces de la zone Nord-Ouest
            for($i = 0; $i < (self::NBROWS)/2; $i++){
                for ($j=0; $j < (self::NBCOLS)/2; $j++) {
                    $t->addPieceQuantik($this->cases[$i][$j]);
                }
            }
        }
        if($dir == self::NE){//les pieces de la zone Nord-Est
            for($i = 0; $i < (self::NBROWS)/2; $i++){
                for ($j=(self::NBCOLS/2); $j < (self::NBCOLS); $j++) {
                    $t->addPieceQuantik($this->cases[$i][$j]);
                }
            }
        }
        if($dir == self::SW){//les pieces de la zone Sud-Ouest
            for($i = (self::NBROWS)/2; $i < (self::NBROWS); $i++){
                for ($j=0; $j <(self::NBCOLS)/2; $j++) {
                    $t->addPieceQuantik($this->cases[$i][$j]);
                }
            }
        }
        if($dir == self::SE){//les pieces de la zone Sud-Est
            for($i = (self::NBROWS)/2 ; $i <self::NBROWS; $i++){
                for ($j=(self::NBCOLS)/2; $j <self::NBCOLS; $j++) {
                    $t->addPieceQuantik($this->cases[$i][$j]);
                }
            }
        }

        return $t;
    }
    //Permet d'afficher les pieces 
    /*public function __toString() : string {
        $s = "";
        $s .= "(";
        for($i = 0; $i < self::NBROWS; $i++){
            for($j = 0; $j < self::NBCOLS; $j++){
                $s .= "(".$this->cases[$i][$j].")";
            }
            $s .= "</br>";
        }
        $s .= ")";
        return $s;
    }*/
    //Permet d'afficher les pieces 
    public function __toString() : string {
        $s = "";
        for($i = 0; $i < self::NBROWS; $i++){
            for($j = 0; $j < self::NBCOLS; $j++){
                $s .= "".$this->cases[$i][$j];
            }
            $s .= "</br>";
        }
        return $s;
    }

    //Cette methode permet d'indiquer la zone ou se situe une piece avec les coordonnées rowNum et colNum
    public static function getCornerFromCoord(int $rowNum, int $colNum) : int{
        //si les coordonnées appartient à la partie en haut à gauche
        if(($rowNum >= 0 && $rowNum < (self::NBROWS/2)) && ($colNum >= 0 && $colNum < (self::NBCOLS/2)) ){
            return self::NW;

            //si les coordonnées appartient à la partie en bas à droite
        }elseif (($rowNum >= (self::NBROWS/2) && $rowNum < self::NBROWS) && ($colNum >= (self::NBCOLS/2) && $colNum < self::NBCOLS )) {
            return self::SE;

            //si les coordonnées appartient à la partie en haut à droite
        }elseif(($rowNum >= 0 && $rowNum < (self::NBROWS/2)) && ($colNum >= (self::NBCOLS/2) && $colNum < self::NBCOLS ) ){
            return self::NE;

            //si les coordonnées appartient à la partie en bas à gauche
        }elseif(  ($rowNum >= (self::NBROWS/2)  && $rowNum < (self::NBROWS)) && ($colNum >= 0 && $colNum < (self::NBCOLS/2))){
            return self::SW;

            //les coordonnées sont érronnées
        }else{
            return -1 ;
        }

    }
}


?>