<?php

require_once "PlateauQuantik.php";
require_once "ArrayPieceQuantik.php";

/**
 * Class QuantikUIGenerator
 */
class QuantikUIGenerator
{

    /**
     * @param string $title
     * @return string
     */

    //methode pour generer le début du squelette d'un fichier HTML jusqu'à la balise <body>
    public static function getDebutHTML(string $title = "Quantik"): string
    {
        return "<!doctype html>
                <html lang='fr'>
                    <head>
                        <meta charset='UTF-8'>
                        <title>$title</title>
                        <link rel=\"stylesheet\" type=\"text/css\" href=\"quantik.css?t=<?php echo time();?\" />
                    </head>
                    <body>
                    <div class='contenu'>
                        
                            <div class='user1'>
                                <img src='./images/firdaous.jpeg' alt='mami'>
                                <h4>Firdaws EL HALAFI</h4>
                            </div>
                        
                        <h1 class=\"quantik1\">$title</h1>
                        <div class='user2'>
                            <img src='./images/mamita.jpeg' alt='mami'>
                            <h4>Mame Fatou SOUNG</h4>
                        </div>
                    </div>
                    <div class='quantik'>\n";
    }

    /**
     * @return string
     */

    //methode pour generer la fin du squelette d'un fichier HTML à partir de la baliser </body> 
    public static function getFinHTML(): string
    {
        return "</div></body>\n</html>";
    }

    /**
     * @param string $message
     * @return string
     */

    //genere la page d'erreur
    public static function getPageErreur(string $message): string
    {
        header("HTTP/1.1 400 Bad Request");
        $resultat = self::getDebutHTML("400 Bad Request");
        $resultat .= "<h2>$message</h2>";
        $resultat .= "<p><br /><br /><br /><a href='quantik.php?reset'>Retour à l'accueil...</a></p>";
        $resultat .= self::getFinHTML();
        return $resultat;
    }

    /**
     * @param PieceQuantik $pq
     * @return string
     */

    public static function getButtonClass(PieceQuantik $pq) {
        if ($pq->getForme()==PieceQuantik::VOID)
            return "vide";
        $ch = $pq->__toString();
        return substr($ch,1,2).substr($ch,4,1);
    }

    //cette methode permet d'inserer les icone dans le plateau et les formulaires à chaque création d'une page
    public static function insererImg(PieceQuantik $piece) :string{
        $resultat ="";
        if ($piece->getForme() == 1 && $piece->getCouleur() == 0) {
                $resultat.="<img src='./images/cube_blanc.png' alt=''>";
            }
            else if ($piece->getForme() == 1 && $piece->getCouleur() == 1) {
                $resultat.="<img src='./images/cube_noir.png' alt=''>";
            }
            else if ($piece->getForme() == 2 && $piece->getCouleur() == 0) {
                $resultat.="<img src='./images/cone_blanc.png' alt=''>";
            }
            else if ($piece->getForme() == 2 && $piece->getCouleur() == 1) {
                $resultat.="<img src='./images/cone_noire.png' alt=''>";
            }
            else if ($piece->getForme() == 3 && $piece->getCouleur() == 0) {
                $resultat.="<img src='./images/cyl_blanc.png' alt=''>";
            }
            else if ($piece->getForme() == 3 && $piece->getCouleur() == 1) {
                $resultat.="<img src='./images/cyl_noir.png' alt=''>";
            }
            else if ($piece->getForme() == 4 && $piece->getCouleur() == 0) {
                $resultat.="<img src='./images/sphere_blanc.png' alt=''>";
            }
            else if ($piece->getForme() == 4 && $piece->getCouleur() == 1) {
                $resultat.="<img src='./images/sphere_noir.png' alt=''>";
            }
            else{
                $resultat.=self::getButtonClass($piece);
            }
            return $resultat;
    }

    /**
     * production d'un bloc HTML pour présenter l'état du plateau de jeu,
     * l'attribution de classes en vue d'une utilisation avec les est un bon investissement...
     * @param PlateauQuantik $p
     * @return string
     */

    //on parcour le plateau et pour chaque piece trouvé on crée un bouton desactivé en y inserant l'icone correspondant
    public static function getDivPlateauQuantik(PlateauQuantik $p): string
    {
        $resultat ="<div class='plateau'>";
        $resultat.="<h3 class='quantik1'>Plateau</h3>";
        
        $resultat.="<table class='tab'>";
        for ($i=0; $i < $p::NBROWS ; $i++){ 
            $resultat.="<tr class='row'>";
            for ($j=0; $j < $p::NBCOLS ; $j++){
                $resultat.="<td class='col'>";
                $resultat.="<button class='bl' type='submit' name='active' disabled >".self::insererImg($p->getPiece($i,$j))."</button>";
                $resultat.="</td>";
            }
            $resultat.="</tr>";
        }
        $resultat.="</table>";
        $resultat.="</div>\n";
        return $resultat;
    }

    /**
     * @param ArrayPieceQuantik $apq
     * @param int $pos permet d'identifier la pièce qui a été sélectionnée par l'utilisateur avant de la poser (si != -1)
     * @return string
     */

    //elle permet d'afficher le pieces disponibles de $apq dans une division html, les piece sont dans des boutons desactivés
    public static function getDivPiecesDisponibles(ArrayPieceQuantik $apq, int $pos = -1): string {
        $resultat ="<div class ='pieces_disponibles'>";
        if ( $apq->getPieceQuantik(0)->getCouleur() == 0 ) 
            $resultat.="<p class='titre'>BLANCS</p>";
        else
            $resultat.="<p class='titre'>NOIRS</p>";
        for($i=0;$i<$apq->getTaille(); $i ++){
            $class = "";
            if ( $pos != -1 && $pos == $i ) 
                $class .= "btn_select";
            $resultat.=" <button class='".$class."' type='submit' name='active' disabled>".self::insererImg($apq->getPieceQuantik($i))."</button>";
        }
        $resultat.="</div></br>";
        return $resultat;
    }

    /**
     * @param ArrayPieceQuantik $apq
     * @return string
     */

    //elle permet d'afficher le pieces de $apq dans un formulaire, les piece sont dans des boutons activés, elle va permettre de selectionner une piece et renvoie sa position
    public static function getFormSelectionPiece(ArrayPieceQuantik $apq): string {
        $resultat = "<form action='".$_SERVER['PHP_SELF']."' method='get'><div class='pieces_disponibles'>";
        if ( $apq->getPieceQuantik(0)->getCouleur() == 0 ) 
            $resultat.="<p class='titre'>BLANCS</p>";
        else
            $resultat.="<p class='titre'>NOIRES</p>";
        for($i =0; $i<$apq->getTaille(); $i++){
                $resultat.=" <button class='choix' type='submit' name='active' value='".$i."'>".self::insererImg($apq->getPieceQuantik($i))."</button>";
        } 
        $resultat.="<p><input type='hidden' value='choisirPiece' name='action' /></p>";

        $resultat.="</div></form>\n";
        return $resultat;
    }

    /**
     * @param PlateauQuantik $plateau
     * @param PieceQuantik $piece
     * @param int $position position de la pièce qui sera posée en vue de la transmettre via un champ caché du formulaire
     * @return string
     */

   //genere le plateau associé à la piece selectionnée, si une piece est selectionnée on parcour toutes les cases du plateau et on active que les cases ou on peut poser la piece selectionné
    public static function getFormPlateauQuantik(PlateauQuantik $plateau, PieceQuantik $piece, int $position): string {
        $resultat ="";
        /* code du formaire de pose associé au plateau */
        $action = new ActionQuantik($plateau);
        $resultat .="<div class='plateau'><h3 class='quantik1'>Plateau</h3>";
        $resultat .= "<form>";
        $resultat .= "<table class='tab'>";
        for ( $i = 0; $i < PlateauQuantik::NBROWS; $i++ ) {
            $resultat .= "<tr>";
            for ($j = 0; $j < PlateauQuantik::NBCOLS; $j++ ) {
                if ( $action->isValidePose($i, $j, $piece)&& $plateau->getPiece($i,$j)->getForme()==0)
                    $resultat .= "<td class='col'><button class='btn_act' type='submit' name='active' value='$i-$j'>".self::getButtonClass($plateau->getPiece($i,$j))."</button></td>";
                else
                    $resultat .= "<td class='col'><button class='btn' type='submit' name='active' disabled>".self::insererImg($plateau->getPiece($i,$j))."</button></td>";
            }
            $resultat .= "</tr>";
        }
        // ajout d'un formulaire pour modifier le choix de la pièce à poser 
        unset($action); 
        $resultat .= "</table>";

        $resultat .= "<input type=\"hidden\" name=\"action\" value=\"poserPiece\" />";
        $resultat .= "<input type=\"hidden\" name=\"positionPiece\" value=\"$position\" />";
        $resultat .="</form>";
        $resultat .="</div>";
        $resultat .= self::getFormBoutonAnnuler();

        return $resultat;
    }

    /**
     * @return string
     */

    //permet de modifier son choix lorsqu'on a choisi un bouton
    public static function getFormBoutonAnnuler() : string {
       
        return "<div class=''><form><button class='btn_annul' type='submit' name='action' value='annulerChoix'>Changer de pièce</button></form></div>";
    }

    /**
     * @param int $couleur
     * @return string
     */

    //permet d'afficher la page qui annonce une victoire lorsqu'un joueur à gagner une partie et propse un lien pour recommencer une nouvelle partie
    public static function getDivMessageVictoire(int $couleur) : string {
        /* div annonçant la couleur victorieuse et proposant un lien pour recommencer une nouvelle partie */
        $resultat ="<div class='victoire'>";
        if ($couleur== PieceQuantik::WHITE) { //si c'est les blancs qui ont gagné 
           $resultat .="<p class='vic'>Victoire pour les BLANCS</p>";
        }
        else{ //si c'est les noirs qui ont gagné
            $resultat .="<p>Victoire pour les NOIRES</p>";
        }
        $resultat .= self::getLienRecommencer();
        $resultat .="</div>";
        return $resultat;
    }

    /**
     * @return string
     */

    //lien pour recommencer une nouvelle partie
    public static function getLienRecommencer():string {
        /* production d'un lien pour recommencer une partie */
        return "<p><a href='?reset'> Recommencer ?</a></p>";
    }

    /**
     * @param array $lesPiecesDispos tableau contenant 2 ArrayPieceQuantik un pour les pièves blanches, un pour les pièces noires
     * @param int $couleurActive
     * @param PlateauQuantik $plateau
     * @return string
     */

    //permet de creer la page de selection de piece
    public static function getPageSelectionPiece(array $lesPiecesDispos, int $couleurActive, PlateauQuantik $plateau): string {
        $pageHTML = QuantikUIGenerator::getDebutHTML(); //debut page html

        /* production de la page permettant de sélectionner une pièce */
        $pageHTML.="<div class='piece'>"; 
       
        $piecesAct = $lesPiecesDispos[$couleurActive]; 
        unset($lesPiecesDispos[$couleurActive]);
        $pieceDesact = array_shift($lesPiecesDispos);

        if ($couleurActive == PieceQuantik::WHITE) { //si c'est aux blancs de jouer
            $pageHTML .= self::getFormSelectionPiece($piecesAct); //on cré un formulaire de choix de piece avec les pieces blanches
            $pageHTML .= self::getDivPiecesDisponibles($pieceDesact); //on cré un formulaire avec des boutons désactivés, avec les pieces noires
        }else{ //si c'est aux noires de jouer
            $pageHTML .= self::getDivPiecesDisponibles($pieceDesact);//on cré un formulaire de choix de piece avec les pieces noires
            $pageHTML .= self::getFormSelectionPiece($piecesAct);//on cré un formulaire avec des boutons désactivés, avec les pieces blanches
        }
        
        $pageHTML.="</div>";
        $pageHTML.=QuantikUIGenerator::getDivPlateauQuantik($plateau); //on cré le plateau

        return $pageHTML. self::getFinHTML();
    }


    /**
     * @param array $lesPiecesDispos tableau contenant 2 ArrayPieceQuantik un pour les pièves blanches, un pour les pièces noires
     * @param int $couleurActive
     * @param int $posSelection position de la pièce sélectionnée dans la couleur active
     * @param PlateauQuantik $plateau
     * @return string
     */

    //permet de creer la page pour poser une piece
    public static function getPagePosePiece(array $lesPiecesDispos, int $couleurActive, int $posSelection, PlateauQuantik $plateau): string {
        $pageHTML = QuantikUIGenerator::getDebutHTML();
           
        $piecesAct = $lesPiecesDispos[$couleurActive];
        array_splice($lesPiecesDispos, $couleurActive, 1);
        $pieceDesact = array_shift($lesPiecesDispos);

        $piecePoser = $piecesAct->getPieceQuantik($posSelection);
        $pageHTML .=  "<div class='piece'>";
        if ($couleurActive == PieceQuantik::WHITE) {
           $pageHTML .= self::getDivPiecesDisponibles($piecesAct, $posSelection);
           $pageHTML .= self::getDivPiecesDisponibles($pieceDesact);
         }
         else{
            $pageHTML .= self::getDivPiecesDisponibles($pieceDesact);
            $pageHTML .= self::getDivPiecesDisponibles($piecesAct, $posSelection);
         }
        $pageHTML .=  "</div>";
         

        $pageHTML .= self::getFormPlateauQuantik($plateau, $piecePoser, $posSelection);
        return $pageHTML . self::getFinHTML();
    }

    /**
     * @param array $lesPiecesDispos tableau contenant 2 ArrayPieceQuantik un pour les pièves blanches, un pour les pièces noires
     * @param int $couleurActive
     * @param int $posSelection
     * @param PlateauQuantik $plateau
     * @return string
     */
    //affiche la page annocant une victoire
    public static function getPageVictoire(array $lesPiecesDispos, int $couleurActive, PlateauQuantik $plateau): string {
        $pageHTML = QuantikUIGenerator::getDebutHTML();       
       
        $pageHTML .= self::getDivMessageVictoire($couleurActive);
        $piecesAct = $lesPiecesDispos[$couleurActive];
        unset($lesPiecesDispos[$couleurActive]);
        $pieceDesact = array_shift($lesPiecesDispos);
        $pageHTML .= "<div class='piece'>";
        if ($couleurActive == PieceQuantik::WHITE) {
            $pageHTML .= self::getDivPiecesDisponibles($piecesAct);
            $pageHTML .= self::getDivPiecesDisponibles($pieceDesact);
        }
        else{
            $pageHTML .= self::getDivPiecesDisponibles($pieceDesact);
            $pageHTML .= self::getDivPiecesDisponibles($piecesAct);
        }
        $pageHTML .= "</div>";  
        $pageHTML .= self::getDivPlateauQuantik($plateau);

        return $pageHTML . self::getFinHTML();

    }

   //elle permet de detecter la présence d'une combinaison lors d'une partie
    public static function checkWin($action) :bool {
        for ( $i = 0; $i < PlateauQuantik::NBROWS; $i++ ) {
        //Le plateau est carré nous pouvons donc vérifier colonne et ligne en même temps.
            if ( $action->isRowWin($i) || $action->isColWin($i) ) {
                return true;
            }
        }
        return ( $action->isCornerWin(PlateauQuantik::NW) ||
             $action->isCornerWin(PlateauQuantik::NE) ||
             $action->isCornerWin(PlateauQuantik::SW) ||
             $action->isCornerWin(PlateauQuantik::SE)
        );
    }

}

?> 