<?php
/**
 * @author Firdaous EL HALAFI et Mame Fatou SOUNG
 * @date janvier 2021
 */

require_once("PieceQuantik.php");
require_once("PlateauQuantik.php");
require_once("ActionQuantik.php");
require_once("QuantikException.php");
require_once("QuantikUIGenerator.php");

session_start();
//session_destroy();

if (isset($_GET['reset'])) { //pratique pour réinitialiser une partie à la main
    unset($_SESSION['etat']);
    unset($_SESSION['lesBlancs']);
    unset($_SESSION['lesNoirs']);
    unset($_SESSION['couleurActive']);
    unset($_SESSION['plateau']);
    unset($_SESSION['message']);
}

if (empty($_SESSION)) { // initialisation des variables de session
    $_SESSION['lesBlancs'] = ArrayPieceQuantik::initPiecesBlanches();
    $_SESSION['lesNoirs'] = ArrayPieceQuantik::initPiecesNoires();
    $_SESSION['plateau'] = new PlateauQuantik();
    $_SESSION['etat'] = 'choixPiece';
    $_SESSION['couleurActive'] = PieceQuantik::WHITE;
    $_SESSION['message'] = "";
}

$pageHTML = "";

$aq = new ActionQuantik($_SESSION['plateau']);

// on réalise les actions correspondant à l'action en cours :
    try {
        if (isset($_GET['action'])) {
            switch ($_GET['action']) {
                case 'choisirPiece': //si on choisi une piece
                    $_SESSION['etat'] = 'posePiece'; //on passeà l'etat pose piece
                    break;
                case 'poserPiece': //si on pose la piece selectinnée 
                    
                    $positionPiece = $_GET['positionPiece'];
                    $arrayPieceQuantik = ($_SESSION['couleurActive'] == PieceQuantik::WHITE)?$_SESSION['lesBlancs']:$_SESSION['lesNoirs'];
                    list($row, $col) = explode("-", $_GET["active"]); //on rassemble les éléments $row et $col sous forme de tableau et on les separe les par des traits
                    if ( !(ctype_digit($row)) ||  !(ctype_digit($col))) throw new QuantikException("Coordonnées invalides"); // on verifie si les valeurs de $row et $col sont des chiffres, sinon une exception sera renvoyée
                    try {
                        $piece = $arrayPieceQuantik->getPieceQuantik($positionPiece);
                        if ( $aq->isValidePose($row, $col, $piece) ) {
                            $aq->posePiece($row, $col, $piece);
                            $arrayPieceQuantik->removePieceQuantik($positionPiece);
                            if ( QuantikUIGenerator::checkWin($aq) || !($aq->checkPosePossible($_SESSION['lesBlancs'])) || !($aq->checkPosePossible($_SESSION['lesNoirs'])) ) { //on teste si y a une victoir ou pas : on gagne s il ya une combinaison gagnante  ou l'adversaire ne peux plus poser de piece
                                $_SESSION['etat'] = 'victoire'; //on passe à la page victoire
                            } 
                            else { //sinon on passe retourne à l'état choixPiece et l'équipe adverse va jouer
                                $_SESSION['couleurActive'] = ($_SESSION['couleurActive'] == PieceQuantik::WHITE)?PieceQuantik::BLACK:PieceQuantik::WHITE;
                                $_SESSION['etat'] = 'choixPiece';
                            }
                    }
                }catch(\Exception $e) {
                    throw new QuantikException($e->getMessage());
                }
                   
                    break;
                case 'annulerChoix': //si on annule un choix 
                    $_SESSION['etat'] = 'choixPiece'; //on retourne à la page de selection de piece
                    break;
                default:
                    throw new QuantikException("Action non valide");
            }
        }
    } catch (QuantikException $exception) {
            $_SESSION['etat'] = 'bug';
            $_SESSION['message'] = $exception->__toString();
        }

$Pieces = array($_SESSION['lesBlancs'], $_SESSION['lesNoirs']);
switch($_SESSION['etat']) {
    case 'choixPiece':
        $pageHTML= QuantikUIGenerator::getPageSelectionPiece($Pieces, $_SESSION['couleurActive'], $_SESSION['plateau']);
        break;
    case 'posePiece':
        $pageHTML = QuantikUIGenerator::getPagePosePiece($Pieces, $_SESSION['couleurActive'], $_GET['active'], $_SESSION['plateau']);
        break;
    case 'victoire':
        $pageHTML = QuantikUIGenerator::getPageVictoire($Pieces, $_SESSION['couleurActive'], $_SESSION['plateau']);
        break;
    default: // sans doute etape=bug
        echo QuantikUIGenerator::getPageErreur($_SESSION['message']);
        exit(1);
}
// seul echo nécessaire toute la pageHTML a été générée dans la variable $pageHTML
echo $pageHTML;


